require 'json'
require 'date'

class NetCity
  SchoolAddress = Struct.new(:state, 
                             :province, 
                             :city, 
                             :func, 
                             :school)

  LoginInfo = Struct.new(:school_address,
                         :user_name,
                         :password)

  class Diary
    include Enumerable

    attr_reader :start, :end

    def initialize(hash)
      @start = hash['weekStart']
      @end   = hash['weekEnd']
      @days  = hash['weekDays'].map { |h| Day.new(h) }
    end

    def [](index)
      @days[index]
    end

    def each(&block)
      @days.each { |lesson| block.call(day) }
    end
  end

  class Day
    include Enumerable

    attr_reader :date

    def initialize(hash)
      @date    = hash['date']
      @lessons = hash['lessons'].map { |h| Lesson.new(h) }
    end

    def [](index)
      @lessons[index]
    end

    def each(&block)
      @lessons.each { |lesson| block.call(lesson) }
    end
  end

  Assignment = Struct.new(:id, 
                          :type, 
                          :name,
                          :mark,
                          :deadline)

  class Lesson
    include Enumerable

    attr_reader :start, :end, :number, :name, :room

    def initialize(hash)
      @start  = hash['startTime']
      @end    = hash['endTime']
      @number = hash['number']
      @name   = hash['subjectName']
      @room   = hash['room']
      return unless hash.include?('assignments')

      @assignments = hash['assignments'].map do |h|
        Assignment.new.tap do |a|
          a.id       = h['id']
          a.type     = h['typeId']
          a.name     = h['assignmentName']
          a.mark     = h['mark']['mark'] if h.include?('mark') 
          a.deadline = h['dueDate']
        end
      end
    end

    def [](index)
      @assignments[index]
    end

    def each(&block)
      @assignments.each { |assignment| block.call(assignment) }
    end
  end
end

module JSON
  def parse_diary(source)
    NetCity::Diary.new(JSON.parse(source))
  end

  module_function :parse_diary
end
